/*
 * Mirai. Copyright (C) 2024 Vyn
 * This file is licensed under version 3 of the GNU General Public License (GPL-3.0-only)
 * The license can be found in the LICENSE file or at https://www.gnu.org/licenses/gpl-3.0.txt
 */

#pragma once

#include "DateTime.h"
#include <optional>
#include <string>
#include <vector>

namespace mirai
{

struct day_data {
	int id;
	Date date;
};

struct updatable_day_data {
	std::optional<Date> date;
};

enum task_state { TODO, DONE };

struct task_data {
	int id;
	std::string title;
	task_state state;
	std::optional<int> dayId;
	std::optional<int> eventId;
};

struct updatable_task_data {
	std::optional<std::string> title = std::nullopt;
	std::optional<task_state> state = std::nullopt;
	std::optional<std::optional<int>> dayId = std::nullopt;
	std::optional<std::optional<int>> eventId = std::nullopt;
};

struct event_data {
	int id;
	int dayId;
	std::string title;
	Time startsAt;
	Time endsAt;
};

struct updatable_event_data {
	std::optional<int> dayId;
	std::optional<std::string> title;
	std::optional<Time> startsAt;
	std::optional<Time> endsAt;
};

class data_provider
{

  public:
	data_provider() {};
	data_provider(data_provider &) = delete;
	data_provider(data_provider &&) = delete;

	virtual ~data_provider() = default;
	virtual void save() = 0;
	virtual void load() = 0;

	virtual std::string name() const = 0;

	// Tasks
	virtual task_data insert_task(const task_data &task) = 0;
	virtual void remove_task_by_id(int taskId) = 0;
	virtual std::optional<task_data> get_task_by_id(int taskId) = 0;
	virtual std::vector<task_data> get_tasks_by_event_id(int eventId) = 0;
	virtual std::vector<task_data> get_tasks_by_day_id(int dayId) = 0;
	virtual std::vector<task_data> get_tasks_by_date(Date date) = 0;
	virtual std::vector<task_data> get_tasks_without_date() = 0;
	virtual std::vector<task_data> get_tasks() = 0;
	virtual void update_task(int taskId, updatable_task_data updateData) = 0;

	// Events
	virtual event_data insertEvent(const event_data &task) = 0;
	virtual void remove_event_by_id(int eventId) = 0;
	virtual std::optional<event_data> get_event_by_id(int eventId) = 0;
	virtual std::vector<event_data> get_events() = 0;
	virtual std::vector<event_data> get_events_by_date(Date date) = 0;
	virtual void update_event(int eventId, updatable_event_data updateData) = 0;

	// Days
	virtual day_data insert_day(const day_data &day) = 0;
	virtual void remove_day_by_id(int dayId) = 0;
	virtual std::vector<day_data> get_days() = 0;
	virtual std::optional<day_data> get_day_by_date(const Date &date) = 0;
	virtual void update_day(int dayId, updatable_day_data updateData) = 0;

	const int id = data_provider::generate_next_id();

	void set_dirty(bool dirty)
	{
		_is_dirty = dirty;
	};

	bool is_dirty() const
	{
		return _is_dirty;
	};

  private:
	bool _is_dirty = false;

	static int generate_next_id()
	{
		static int nextId = 0;
		return nextId++;
	}
};

} // namespace mirai
