/*
 * Mirai. Copyright (C) 2024 Vyn
 * This file is licensed under version 3 of the GNU General Public License (GPL-3.0-only)
 * The license can be found in the LICENSE file or at https://www.gnu.org/licenses/gpl-3.0.txt
 */

#pragma once

#include "DataProvider.h"
#include "DateTime.h"
#include "Day.h"
#include "Event.h"
#include "Task.h"
#include <string>
#include <vector>

namespace mirai
{

struct createTaskParams {
	std::string title;
	std::optional<Event> event;
	std::optional<Date> date;
};

struct createEventParams {
	std::string title;
	Date date;
	Time startsAt;
	Time endsAt;
};

struct SourceConstructor {
	std::string name;
	std::string color;
	data_provider *sourceDataProvider;
};

class source
{
  public:
	source(SourceConstructor params)
		: data(params.sourceDataProvider), _name(params.name), _color(params.color),
		  _type("FileSystemMarkdown")
	{
	}

	source(source &) = delete;
	source(source &&) = delete;
	source operator=(source &) = delete;
	source operator=(source &&) = delete;

	~source() = default;

	void init();
	void save();
	void load();

	std::string name() const;
	std::string type() const;
	std::string color() const;
	class data_provider *data_provider();

	void set_name(const std::string &name);
	void set_color(const std::string &color);

	void create_task(const createTaskParams &task);
	void remove_task(const Task &task);
	std::optional<Task> get_task_by_id(int taskId);
	std::vector<Task> get_tasks_by_date(Date date);
	std::vector<Task> get_unscheduled_tasks();
	std::vector<Task> get_tasks();

	void create_event(const createEventParams &eventToCreate);
	void remove_event(const Event &event);
	std::vector<Event> get_events();
	std::optional<Event> get_event_by_id(int eventId);
	std::vector<Event> get_events_by_date(Date date);

	std::vector<Day> get_days();
	std::optional<Day> get_day_by_date(Date date);

	const int id = source::generate_next_id();

	bool is_dirty() const
	{
		return data->is_dirty();
	};

  private:
	static int generate_next_id()
	{
		static int nextId = 0;
		return nextId++;
	}

	std::string _name;
	std::string _type;
	std::string _color;
	class data_provider *data;
};
} // namespace mirai
