/*
 * Mirai. Copyright (C) 2024 Vyn
 * This file is licensed under version 3 of the GNU General Public License (GPL-3.0-only)
 * The license can be found in the LICENSE file or at https://www.gnu.org/licenses/gpl-3.0.txt
 */

#include "MarkdownDataProvider.h"
#include "DataProvider.h"
#include "cpp-utils/debug.h"
#include "utils.h"
#include <algorithm>
#include <cassert>
#include <functional>
#include <optional>
#include <regex>
#include <vector>

namespace mirai
{

std::string markdown_data_provider::path() const
{
	return _file_path;
}

std::string markdown_data_provider::name() const
{
	return _data.name;
}

void markdown_data_provider::save()
{
	std::ofstream file(_file_path);
	if (!file.is_open()) {
		throw std::runtime_error("can't create " + _file_path);
	}

	const std::string content = to_markdown();

	file << content;
	file.close();
	set_dirty(false);
};

void markdown_data_provider::load()
{
	std::ifstream file(_file_path);
	if (!file.is_open()) {
		return;
	}
	std::string content = "";
	std::string line;

	while (std::getline(file, line)) {
		content += line + "\n";
	}
	file.close();
	_data = parse_markdown(content);
};

task_data markdown_data_provider::insert_task(const task_data &taskData)
{
	_data.tasks.push_back(taskData);
	set_dirty(true);
	return taskData;
}

void markdown_data_provider::update_task(int taskId, updatable_task_data updateData)
{
	auto taskData = ptrFindFirst(_data.tasks, [&](const task_data &task) {
		return task.id == taskId;
	});
	assert(taskData != nullptr); // Shouldn't be possible
	if (updateData.title.has_value()) {
		taskData->title = updateData.title.value();
	}
	if (updateData.state.has_value()) {
		taskData->state = updateData.state.value();
	}
	if (updateData.dayId.has_value()) {
		taskData->dayId = updateData.dayId.value();
	}
	if (updateData.eventId.has_value()) {
		taskData->eventId = updateData.eventId.value();
	}
	set_dirty(true);
}

std::optional<task_data> markdown_data_provider::get_task_by_id(int taskId)
{
	return findFirst(_data.tasks, [&](const task_data &task) {
		return task.id == taskId;
	});
}

std::vector<task_data> markdown_data_provider::get_tasks_by_event_id(int eventId)
{
	return findAll(_data.tasks, [&](const task_data &task) {
		return task.eventId == eventId;
	});
}

std::vector<task_data> markdown_data_provider::get_tasks_by_day_id(int dayId)
{
	return findAll(_data.tasks, [&](const task_data &task) {
		return task.dayId == dayId;
	});
}

std::vector<task_data> markdown_data_provider::get_tasks_without_date()
{
	return findAll(_data.tasks, [](const task_data &t) {
		return t.dayId == std::nullopt && t.eventId == std::nullopt;
	});
}

std::vector<task_data> markdown_data_provider::get_tasks_by_date(Date date)
{
	auto day = get_day_by_date(date);
	if (!day.has_value()) {
		return {};
	}
	return findAll(_data.tasks, [&](const task_data &t) {
		return t.dayId == day.value().id;
	});
}

std::vector<task_data> markdown_data_provider::get_tasks()
{
	return _data.tasks;
}

void markdown_data_provider::remove_task_by_id(int taskId)
{
	_data.tasks.erase(
		std::remove_if(
			_data.tasks.begin(), _data.tasks.end(),
			[&](const task_data &task) {
				return task.id == taskId;
			}
		),
		_data.tasks.end()
	);
	set_dirty(true);
}

day_data markdown_data_provider::insert_day(const day_data &dayData)
{
	_data.days.push_back(dayData);
	set_dirty(true);
	return dayData;
}

void markdown_data_provider::update_day(int dayId, updatable_day_data updateData)
{
	auto dayData = ptrFindFirst(_data.days, [&](const day_data &day) {
		return day.id == dayId;
	});
	assert(dayData != nullptr); // Shouldn't be possible
	if (updateData.date.has_value()) {
		dayData->date = updateData.date.value();
	}
	set_dirty(true);
}

void markdown_data_provider::remove_day_by_id(int dayId)
{
	_data.days.erase(
		std::remove_if(
			_data.days.begin(), _data.days.end(),
			[&](const day_data &day) {
				return day.id == dayId;
			}
		),
		_data.days.end()
	);
	set_dirty(true);
}

std::vector<day_data> markdown_data_provider::get_days()
{
	return _data.days;
}

std::optional<day_data> markdown_data_provider::get_day_by_date(const Date &date)
{
	return findFirst(_data.days, [&](const day_data &day) {
		return day.date == date;
	});
}

event_data markdown_data_provider::insertEvent(const event_data &eventData)
{
	_data.events.push_back(eventData);
	set_dirty(true);
	return eventData;
}

void markdown_data_provider::update_event(int eventId, updatable_event_data updateData)
{
	auto eventData = ptrFindFirst(_data.events, [&](const event_data &event) {
		return event.id == eventId;
	});
	assert(eventData != nullptr); // Shouldn't be possible
	if (updateData.title.has_value()) {
		eventData->title = updateData.title.value();
	}
	if (updateData.dayId.has_value()) {
		eventData->dayId = updateData.dayId.value();
	}
	if (updateData.startsAt.has_value()) {
		eventData->startsAt = updateData.startsAt.value();
	}
	if (updateData.endsAt.has_value()) {
		eventData->endsAt = updateData.endsAt.value();
	}
	set_dirty(true);
}

void markdown_data_provider::remove_event_by_id(int eventId)
{
	_data.events.erase(
		std::remove_if(
			_data.events.begin(), _data.events.end(),
			[&](const event_data &event) {
				return event.id == eventId;
			}
		),
		_data.events.end()
	);
	set_dirty(true);
}

std::optional<event_data> markdown_data_provider::get_event_by_id(int eventId)
{
	return findFirst(_data.events, [&](const event_data &event) {
		return event.id == eventId;
	});
}

std::vector<event_data> markdown_data_provider::get_events_by_date(Date date)
{
	auto day = get_day_by_date(date);
	if (!day.has_value()) {
		return {};
	}
	return findAll(_data.events, [&](const event_data &event) {
		return event.dayId == day.value().id;
	});
}

std::vector<event_data> markdown_data_provider::get_events()
{
	return _data.events;
}
} // namespace mirai
