/*
 * Mirai. Copyright (C) 2024 Vyn
 * This file is licensed under version 3 of the GNU General Public License (GPL-3.0-only)
 * The license can be found in the LICENSE file or at https://www.gnu.org/licenses/gpl-3.0.txt
 */

#include "Mirai.h"
#include "DataProvider.h"
#include "MarkdownDataProvider.h"
#include "Source.h"
#include "rei-json/Array.h"
#include "rei-json/Object.h"
#include "rei-json/json.h"
#include <algorithm>
#include <cassert>
#include <iostream>
#include <memory>
#include <print>
#include <string>
#include <vector>

namespace mirai
{

void Mirai::loadConfig(const std::string &path)
{
	std::ifstream file(path);
	if (!file) {
		return;
	}
	std::stringstream buffer;
	buffer << file.rdbuf();
	auto jsonStr = buffer.str();

	auto configJson = rei::json::parse(jsonStr);

	assert(configJson.isObject());
	assert(configJson.asObject()["files"].isArray());
	auto jsonSources = configJson.asObject()["files"].asArray();
	for (int i = 0; i < jsonSources.count(); ++i) {
		auto &filePath = jsonSources[i];
		assert(filePath.isObject());
		const auto name = filePath.asObject().getString("name");
		const auto color = filePath.asObject().getString("color");
		const auto path = filePath.asObject().getString("path");
		std::unique_ptr<data_provider> file = std::make_unique<markdown_data_provider>(path);
		data_provider *sourceDataProvider = file.release();
		sourceDataProvider->load();
		sources_.push_back(
			std::make_unique<source>(SourceConstructor{
				.name = name, .color = color, .sourceDataProvider = sourceDataProvider
			})
		);
	}
}

void Mirai::saveConfig()
{
	std::ofstream file(configPath_);
	if (!file.is_open()) {
		std::print(std::cerr, "Can't save config to {}", configPath_);
		return;
	}
	auto files = rei::json::JsonArray{};
	auto configJson = rei::json::JsonObject{};
	configJson.set("files", files);
	for (auto &source : sources_) {

		rei::json::JsonObject jsonSource;
		jsonSource.set("name", source->name());
		jsonSource.set("color", source->color());
		auto dataProvider = dynamic_cast<markdown_data_provider *>(source->data_provider());
		jsonSource.set("path", dataProvider->path());
		jsonSource.set("type", "FileSystemMarkdown");
		configJson.getArray("files").push(jsonSource);
	}
	file << rei::json::toString(configJson);
	file.close();
}

Mirai::Mirai(const std::string &configFilePath) : configPath_(configFilePath)
{
	loadConfig(configFilePath);
}

void Mirai::addSource(
	const std::string &name, const std::string &type, std::unique_ptr<data_provider> &&new_source
)
{
	data_provider *sourceDataProvider = new_source.release();
	sourceDataProvider->load();
	sources_.push_back(
		std::make_unique<source>(
			SourceConstructor{.name = name, .sourceDataProvider = sourceDataProvider}
		)
	);
	saveConfig();
	sourceAdded.emit(nullptr);
};

void Mirai::editSource(
	int id, const std::string &name, const std::string &color, const std::string &path
)
{
	auto source = getSourceById(id);
	source->set_name(name);
	source->set_color(color);

	data_provider *sourceDataProvider = source->data_provider();
	saveConfig();
	sourceEdited.emit(nullptr);
}

void Mirai::deleteSource(int id)
{
	sources_.erase(
		std::remove_if(
			sources_.begin(), sources_.end(),
			[&](const std::unique_ptr<source> &source) {
				return source->id == id;
			}
		),
		sources_.end()
	);

	saveConfig();
	sourceDeleted.emit(id);
}

void Mirai::unloadAllSources()
{
	sources_.clear();
}

void Mirai::save()
{
	for (auto &source : sources_) {
		if (source->is_dirty()) {
			source->save();
		}
	}
}

std::vector<std::unique_ptr<source>> &Mirai::getSources()
{
	return sources_;
}

source *Mirai::getSourceById(int id)
{
	auto source_found = std::ranges::find_if(sources_, [&](const std::unique_ptr<source> &source) {
		return source->id == id;
	});
	assert(source_found != sources_.end()); // This should not happen
	if (source_found == sources_.end()) {
		return nullptr;
	}
	return source_found->get();
}

source *Mirai::getSourceByName(const std::string &name)
{
	auto source_found = std::ranges::find_if(sources_, [&](const std::unique_ptr<source> &source) {
		return source->name() == name;
	});
	assert(source_found != sources_.end()); // This should not happen
	if (source_found == sources_.end()) {
		return nullptr;
	}
	return source_found->get();
}

void Mirai::onSourceAdded(std::function<void(source *)> f)
{
	sourceAdded.registerCallback(f);
}

void Mirai::onSourceEdited(std::function<void(source *)> f)
{
	sourceEdited.registerCallback(f);
}

void Mirai::onSourceDeleted(std::function<void(int)> f)
{
	sourceDeleted.registerCallback(f);
}

} // namespace mirai
