/*
 * Mirai. Copyright (C) 2024 Vyn
 * This file is licensed under version 3 of the GNU General Public License (GPL-3.0-only)
 * The license can be found in the LICENSE file or at https://www.gnu.org/licenses/gpl-3.0.txt
 */

#include "Source.h"
#include "DataProvider.h"
#include "utils.h"
#include <algorithm>
#include <iterator>
#include <optional>
#include <string>
#include <vector>

namespace mirai
{

void source::save()
{
	data->save();
}

void source::create_task(const createTaskParams &task)
{
	std::optional<int> dayId = std::nullopt;
	std::optional<int> eventId = std::nullopt;

	if (!task.event.has_value() && task.date.has_value()) {
		auto day = data->get_day_by_date(task.date.value());
		if (!day.has_value() && task.date.has_value()) {
			day = data->insert_day({.id = generateUniqueId(), .date = task.date.value()});
		}
		dayId = day->id;
	}

	if (task.event.has_value() && task.event->id()) {
		eventId = task.event.value().id();
	}

	data->insert_task({
		.id = generateUniqueId(),
		.title = task.title,
		.state = TODO,
		.dayId = dayId,
		.eventId = eventId,
	});
};

std::vector<Day> source::get_days()
{
	auto daysData = data->get_days();
	std::vector<Day> days;
	std::transform(
		daysData.begin(), daysData.end(), std::back_inserter(days), [&](const day_data &dayData) {
			return Day{data, dayData};
		}
	);
	return days;
}

std::optional<Day> source::get_day_by_date(Date date)
{
	auto day = data->get_day_by_date(date);
	if (!day.has_value()) {
		return std::nullopt;
	}
	return Day{data, day.value()};
}

void source::create_event(const createEventParams &eventToCreate)
{
	std::optional<int> dayId = std::nullopt;

	auto day = data->get_day_by_date(eventToCreate.date);
	if (!day.has_value()) {
		day = data->insert_day({.id = generateUniqueId(), .date = eventToCreate.date});
		dayId = day->id;
	}

	data->insertEvent({
		.id = generateUniqueId(),
		.dayId = day.value().id,
		.title = eventToCreate.title,
		.startsAt = eventToCreate.startsAt,
		.endsAt = eventToCreate.endsAt,
	});
};

std::optional<Event> source::get_event_by_id(int evendId)
{
	auto event = data->get_event_by_id(evendId);
	if (!event.has_value()) {
		return std::nullopt;
	}
	return Event{data, event.value()};
}

void source::remove_event(const Event &event)
{
	data->remove_event_by_id(event.id());
}

void source::remove_task(const Task &task)
{
	data->remove_task_by_id(task.id());
}

std::optional<Task> source::get_task_by_id(int taskId)
{
	auto taskData = data->get_task_by_id(taskId);
	if (!taskData.has_value()) {
		return std::nullopt;
	}
	Task task{data, taskData.value()};
	return task;
}

std::vector<Task> source::get_tasks()
{
	auto tasksData = data->get_tasks();
	std::vector<Task> tasks;
	std::transform(
		tasksData.begin(), tasksData.end(), std::back_inserter(tasks),
		[&](const task_data &taskData) {
			return Task{data, taskData};
		}
	);
	return tasks;
}

std::vector<Task> source::get_unscheduled_tasks()
{
	auto tasksData = data->get_tasks_without_date();
	std::vector<Task> tasks;
	std::transform(
		tasksData.begin(), tasksData.end(), std::back_inserter(tasks),
		[&](const task_data &taskData) {
			return Task{data, taskData};
		}
	);
	return tasks;
}

std::string source::name() const
{
	return _name;
}

std::string source::color() const
{
	return _color;
}

void source::set_name(const std::string &name)
{
	_name = name;
}

void source::set_color(const std::string &color)
{
	_color = color;
}

std::string source::type() const
{
	// There is only 1 type for now
	return _type;
}

data_provider *source::data_provider()
{
	return data;
}
} // namespace mirai
